﻿using System;
using System.IO;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;

public partial class _Default : System.Web.UI.Page
{
	protected void Page_Load(object sender, EventArgs e)
	{
		
		// bind the list of images from the database
		if (!IsPostBack)
			repImages.DataSource = DataAccess.ListImages();

		DataBind();
	}	

	/// <summary>
	/// Handle the submit button
	/// </summary>
	/// <param name="sender"></param>
	/// <param name="e"></param>
	protected void Submit_Click(object sender, EventArgs e)
	{
		if (IsPostBack)
			HandleUploadedFile();
	}

	/// <summary>
	/// Method to handle pulling the files from the request and send them to the database
	/// </summary>
	private void HandleUploadedFile()
	{
		// get the file instance
		HttpPostedFile fi = Request.Files.Get(0);

		// get the description
		string description = Description.Text;

		// create a byte array to store the file bytes
		byte[] fileBytes = new byte[fi.ContentLength];

		// fill the byte array
		using (System.IO.Stream stream = fi.InputStream)
		{
			stream.Read(fileBytes, 0, fi.ContentLength);
		}

		// insert the image
		DataAccess.InsertImage(description, fileBytes);

		// bind the image list
		repImages.DataSource = DataAccess.ListImages();
		DataBind();

		// clean up
		Description.Text = "";
		fileBytes = null;
	}


	/// <summary>
	/// on each item in the repeater setup the asp controls with some properties
	/// </summary>
	/// <param name="sender"></param>
	/// <param name="e"></param>
	protected void repImages_ItemDataBound(object sender, RepeaterItemEventArgs e)
	{
		// get our object from the datasource
		DataAccess.ImageData item = (DataAccess.ImageData)e.Item.DataItem;

		// get a reference to our controls in the item template
		Image image = (Image)e.Item.FindControl("imageThumb");
		Label label = (Label)e.Item.FindControl("labelDescription");

		// set the image control properties
		if (image != null)
		{
			image.ImageUrl = string.Format("~/Thumbnail.ashx?id={0}", item.ID);
			image.AlternateText = item.Description;
			image.Visible = true;
		}

		// set the image title
		if (label != null)
		{
			label.Text = item.Description;
			label.Visible = true;
		}
	}
}