﻿<%@ WebHandler Language="VB" Class="Thumbnail" %>

Imports System
Imports System.Drawing
Imports System.Drawing.Drawing2D
Imports System.Drawing.Imaging
Imports System.IO
Imports System.Web

Public Class Thumbnail : Implements IHttpHandler
    
    Public Sub ProcessRequest(ByVal context As HttpContext) Implements IHttpHandler.ProcessRequest
        
        ' Set up the response settings
        context.Response.ContentType = "image/jpeg"
        context.Response.Cache.SetCacheability(HttpCacheability.Public)
        context.Response.BufferOutput = False
        
        If Not String.IsNullOrEmpty(context.Request.QueryString("ID")) Then
            
            ' get the id for the image
            Dim id As Integer = Convert.ToInt32(context.Request.QueryString("ID"))
            
            ' get the image as a byte array
            Dim imageAsBytes() As Byte = DataAccess.GetImageByID(id)
            
            ' resize the image to a thumb (100)
            imageAsBytes = ResizeImageFile(imageAsBytes, 100)
            
            ' put the byte array into a stream
            Dim stream As Stream = New MemoryStream(CType(imageAsBytes, Byte()))
            
            ' setup the chunking of the image data (good for large images)
            Dim buffersize As Integer = (1024 * 16)
            Dim buffer As Byte() = New Byte((buffersize) - 1) {}
            
            ' push the bytes to the output stream in chunks
            Dim count As Integer = stream.Read(buffer, 0, buffersize)
            
            While (count > 0)
                context.Response.OutputStream.Write(buffer, 0, count)
                count = stream.Read(buffer, 0, buffersize)
                
            End While
        End If
    End Sub
 
    Public ReadOnly Property IsReusable() As Boolean Implements IHttpHandler.IsReusable
        Get
            Return False
        End Get
    End Property

    ''' <summary> 
    ''' Resizes an image 
    ''' </summary> 
    ''' <param name="imageFile">the byte array of the file</param> 
    ''' <param name="targetSize">the target size of the file (may affect width or height) 
    ''' depends on orientation of file (landscape or portrait)</param> 
    ''' <returns>Byte array containing the resized file</returns> 
    Private Shared Function ResizeImageFile(ByVal imageFile As Byte(), ByVal targetSize As Integer) As Byte()
        
        Using oldImage As System.Drawing.Image = System.Drawing.Image.FromStream(New MemoryStream(imageFile))
            Dim newSize As Size = CalculateDimensions(oldImage.Size, targetSize)
            Using newImage As New Bitmap(newSize.Width, newSize.Height, PixelFormat.Format24bppRgb)
                Using canvas As Graphics = Graphics.FromImage(newImage)
                    canvas.SmoothingMode = SmoothingMode.AntiAlias
                    canvas.InterpolationMode = InterpolationMode.HighQualityBicubic
                    canvas.PixelOffsetMode = PixelOffsetMode.HighQuality
                    canvas.DrawImage(oldImage, New Rectangle(New Point(0, 0), newSize))
                    Dim m As New MemoryStream()
                    newImage.Save(m, ImageFormat.Jpeg)
                    Return m.GetBuffer()
                End Using
            End Using
        End Using
    End Function

    ''' <summary> 
    ''' Calculates the new size of the image based on the target size 
    ''' </summary> 
    ''' <param name="oldSize">Is the size of the original file</param> 
    ''' <param name="targetSize">Is the target size of the resized file</param> 
    ''' <returns>The new size</returns> 
    Private Shared Function CalculateDimensions(ByVal oldSize As Size, ByVal targetSize As Integer) As Size
        
        Dim newSize As New Size()
        
        If oldSize.Height > oldSize.Width Then
            newSize.Width = CInt((oldSize.Width * (CSng(targetSize) / CSng(oldSize.Height))))
            newSize.Height = targetSize
        Else
            newSize.Width = targetSize
            newSize.Height = CInt((oldSize.Height * (CSng(targetSize) / CSng(oldSize.Width))))
        End If
        Return newSize
    End Function
End Class